﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.SignalR;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Helpers;

namespace Tsi1.Api.SignalR
{
    [Authorize]
    public class VideoHub : Hub
    {
        public override async Task OnConnectedAsync()
        {
            await Groups.AddToGroupAsync(Context.ConnectionId, Context.ConnectionId);

            await Clients.Caller.SendAsync(Context.ConnectionId);
        }

        [Authorize(Roles = UserTypes.Professor)]
        public async Task CreateRoom(string roomName)
        {
            await Groups.AddToGroupAsync(Context.ConnectionId, roomName);

            await Clients.Caller.SendAsync("CreateRoom", roomName);
        }

        public async Task Join(string roomName, string offer)
        {
            await Clients.Group(roomName).SendAsync("Join", Context.ConnectionId, offer);

            await Groups.AddToGroupAsync(Context.ConnectionId, roomName);
        }

        public async Task Leave(string roomName)
        {
            await Groups.RemoveFromGroupAsync(Context.ConnectionId, roomName);

            await Clients.Group(roomName).SendAsync("Leave", Context.ConnectionId);
        }

        public async Task NotifyUser(string connectionId, string answer)
        {
            await Clients.Group(connectionId).SendAsync("NotifyUser", Context.ConnectionId, answer);
        }

        public async Task ICECandidate(string connectionId, string candidate)
        {
            await Clients.Group(connectionId).SendAsync("ICECandidate", Context.ConnectionId, candidate);
        }
    }
}
