﻿using AutoMapper;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;
using Tsi1.DataLayer;
using Tsi1.DataLayer.Entities;

namespace Tsi1.BusinessLayer.Services
{
    public class FileService : IFileService
    {
        private readonly Tsi1Context _context;
        private readonly IMapper _mapper;
        private readonly string _path;

        public FileService(Tsi1Context context, IMapper mapper, IHostingEnvironment hostingEnvironment)
        {
            _context = context;
            _mapper = mapper;
            _path = Path.Combine(hostingEnvironment.ContentRootPath, "StaticFiles");
        }

        public async Task<ServiceResult<string>> Create(IFormFile file, string tenantId, string courseId)
        {
            var result = new ServiceResult<string>();

            var folder = Path.Combine(_path, tenantId, courseId);
            Directory.CreateDirectory(folder);

            var fileName = GenerateFileName(file);
            var relativePath = Path.Combine(tenantId, courseId, fileName);
            var filePath = Path.Combine(_path, relativePath);

            var fileStream = new FileStream(filePath, FileMode.Create);
            try
            {
                await file.CopyToAsync(fileStream);
            }
            catch (Exception)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.ErrorSavingFile, filePath);
                return result;
            }
            finally
            {
                fileStream.Close();
            }

            result.Data = relativePath;
            return result;
        }

        public ServiceResult<bool> DeleteFile(string filePath)
        {
            var result = new ServiceResult<bool>();
            
            try
            {
                using (var fileStream = new FileStream(filePath, FileMode.Open, FileAccess.Read, FileShare.None, 4096, FileOptions.DeleteOnClose)) {}
            }
            catch (Exception)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.ErrorDeletingFile, filePath);
                return result;
            }

            return result;
        }

        public bool ExistFile(string relativePath)
        {
            var path = Path.Combine(_path, relativePath);
            return System.IO.File.Exists(path);
        }

        public async Task<List<string>> GetFileNames()
        {
            return await _context.Files.Select(x => x.Path).ToListAsync();
        }

        public List<string> GetPhysicalFiles()
        {
            var fullPaths = Directory.GetFiles(_path, "*", SearchOption.AllDirectories);
            return fullPaths.Select(x => Path.GetRelativePath(_path, x)).ToList();
        }

        private string GenerateFileName(IFormFile file)
        {
            var ext = file.FileName.Split(".").Last();
            var fileName = Guid.NewGuid().ToString();
            return fileName + "." + ext;
        }

    }
}
