﻿using AutoMapper;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;
using Tsi1.DataLayer;
using Tsi1.DataLayer.Entities;

namespace Tsi1.BusinessLayer.Services
{
    public class UserTypeService : IUserTypeService
    {
        private readonly Tsi1Context _context;

        private readonly IMapper _mapper;

        public UserTypeService(Tsi1Context context, IMapper mapper)
        {
            _context = context;
            _mapper = mapper;
        }

        public async Task<ServiceResult<List<UserTypeDto>>> GetAll(string userType)
        {
            var result = new ServiceResult<List<UserTypeDto>>();

            List<UserType> userTypes = null; 

            if (userType == UserTypes.UdelarAdmin)
            {
                userTypes = await _context.UserTypes
                    .Where(x => x.Name == UserTypes.UdelarAdmin ||
                    x.Name == UserTypes.FacultyAdmin)
                    .ToListAsync();
            }
            else if (userType == UserTypes.FacultyAdmin)
            {
                userTypes = await _context.UserTypes
                    .Where(x => x.Name == UserTypes.Student ||
                    x.Name == UserTypes.Professor)
                    .ToListAsync();
            }

            result.Data = _mapper.Map<List<UserTypeDto>>(userTypes);

            return result;
        }

        public async Task<ServiceResult<UserTypeDto>> GetById(int id)
        {
            var result = new ServiceResult<UserTypeDto>();

            var userType = await _context.UserTypes.FirstOrDefaultAsync(x => x.Id == id);

            if (userType == null)
            {
                result.HasError = true;
                result.Message = string.Format(ErrorMessages.UserTypeDoesNotExist, id);
            }
            
            result.Data = _mapper.Map<UserTypeDto>(userType);

            return result;
        }
    }
}
