﻿using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;
using System;
using System.Collections.Generic;
using System.Text;
using Tsi1.DataLayer.Entities;

namespace Tsi1.DataLayer.EntityConfiguration
{
    public class UserConfiguration : IEntityTypeConfiguration<User>
    {
        public void Configure(EntityTypeBuilder<User> builder)
        {
            builder.HasKey(x => x.Id);

            builder.HasIndex(x => new { x.Username, x.TenantId })
                .IsUnique();

            builder.Property(x => x.Username)
                .IsRequired()
                .HasColumnType("character varying(50)");

            builder.Property(x => x.Password)
                .IsRequired()
                .HasColumnType("character varying(255)");

            builder.Property(x => x.FirstName)
                .IsRequired()
                .HasColumnType("character varying(255)");

            builder.Property(x => x.LastName)
                .IsRequired()
                .HasColumnType("character varying(255)");

            builder.Property(x => x.Email)
                .IsRequired()
                .HasColumnType("character varying(255)");

            builder.HasOne(x => x.Student)
                .WithOne(x => x.User)
                .HasForeignKey<User>(x => x.StudentId);

            builder.HasOne(x => x.Professor)
                .WithOne(x => x.User)
                .HasForeignKey<User>(x => x.ProfessorId);

            builder.HasOne(x => x.Tenant)
                .WithMany(x => x.Users)
                .HasForeignKey(x => x.TenantId);
        }
    }
}
