﻿using Microsoft.EntityFrameworkCore.Migrations;
using Npgsql.EntityFrameworkCore.PostgreSQL.Metadata;

namespace Tsi1.DataLayer.Migrations
{
    public partial class section_items : Migration
    {
        protected override void Up(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropForeignKey(
                name: "FK_Forums_Courses_CourseId",
                table: "Forums");

            migrationBuilder.DropIndex(
                name: "IX_Forums_CourseId_Name",
                table: "Forums");

            migrationBuilder.AlterColumn<int>(
                name: "CourseId",
                table: "Forums",
                nullable: true,
                oldClrType: typeof(int),
                oldType: "integer");

            migrationBuilder.AddColumn<int>(
                name: "SectionItemId",
                table: "Forums",
                nullable: false,
                defaultValue: 0);

            migrationBuilder.AddColumn<int>(
                name: "SectionItemId",
                table: "Files",
                nullable: false,
                defaultValue: 0);

            migrationBuilder.CreateTable(
                name: "SectionItemTypes",
                columns: table => new
                {
                    Id = table.Column<int>(nullable: false)
                        .Annotation("Npgsql:ValueGenerationStrategy", NpgsqlValueGenerationStrategy.IdentityByDefaultColumn),
                    Name = table.Column<string>(type: "character varying(100)", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_SectionItemTypes", x => x.Id);
                });

            migrationBuilder.CreateTable(
                name: "Sections",
                columns: table => new
                {
                    Id = table.Column<int>(nullable: false)
                        .Annotation("Npgsql:ValueGenerationStrategy", NpgsqlValueGenerationStrategy.IdentityByDefaultColumn),
                    Name = table.Column<string>(type: "character varying(255)", nullable: false),
                    Order = table.Column<int>(nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_Sections", x => x.Id);
                });

            migrationBuilder.CreateTable(
                name: "SectionItems",
                columns: table => new
                {
                    Id = table.Column<int>(nullable: false)
                        .Annotation("Npgsql:ValueGenerationStrategy", NpgsqlValueGenerationStrategy.IdentityByDefaultColumn),
                    Order = table.Column<int>(nullable: false),
                    SectionId = table.Column<int>(nullable: false),
                    SectionItemTypeId = table.Column<int>(nullable: false),
                    ForumId = table.Column<int>(nullable: true),
                    FileId = table.Column<int>(nullable: true)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_SectionItems", x => x.Id);
                    table.ForeignKey(
                        name: "FK_SectionItems_Files_FileId",
                        column: x => x.FileId,
                        principalTable: "Files",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Restrict);
                    table.ForeignKey(
                        name: "FK_SectionItems_Forums_ForumId",
                        column: x => x.ForumId,
                        principalTable: "Forums",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Restrict);
                    table.ForeignKey(
                        name: "FK_SectionItems_Sections_SectionId",
                        column: x => x.SectionId,
                        principalTable: "Sections",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "FK_SectionItems_SectionItemTypes_SectionItemTypeId",
                        column: x => x.SectionItemTypeId,
                        principalTable: "SectionItemTypes",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateIndex(
                name: "IX_Forums_CourseId",
                table: "Forums",
                column: "CourseId");

            migrationBuilder.CreateIndex(
                name: "IX_SectionItems_FileId",
                table: "SectionItems",
                column: "FileId",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "IX_SectionItems_ForumId",
                table: "SectionItems",
                column: "ForumId",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "IX_SectionItems_SectionId",
                table: "SectionItems",
                column: "SectionId");

            migrationBuilder.CreateIndex(
                name: "IX_SectionItems_SectionItemTypeId",
                table: "SectionItems",
                column: "SectionItemTypeId");

            migrationBuilder.AddForeignKey(
                name: "FK_Forums_Courses_CourseId",
                table: "Forums",
                column: "CourseId",
                principalTable: "Courses",
                principalColumn: "Id",
                onDelete: ReferentialAction.Restrict);
        }

        protected override void Down(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropForeignKey(
                name: "FK_Forums_Courses_CourseId",
                table: "Forums");

            migrationBuilder.DropTable(
                name: "SectionItems");

            migrationBuilder.DropTable(
                name: "Sections");

            migrationBuilder.DropTable(
                name: "SectionItemTypes");

            migrationBuilder.DropIndex(
                name: "IX_Forums_CourseId",
                table: "Forums");

            migrationBuilder.DropColumn(
                name: "SectionItemId",
                table: "Forums");

            migrationBuilder.DropColumn(
                name: "SectionItemId",
                table: "Files");

            migrationBuilder.AlterColumn<int>(
                name: "CourseId",
                table: "Forums",
                type: "integer",
                nullable: false,
                oldClrType: typeof(int),
                oldNullable: true);

            migrationBuilder.CreateIndex(
                name: "IX_Forums_CourseId_Name",
                table: "Forums",
                columns: new[] { "CourseId", "Name" },
                unique: true);

            migrationBuilder.AddForeignKey(
                name: "FK_Forums_Courses_CourseId",
                table: "Forums",
                column: "CourseId",
                principalTable: "Courses",
                principalColumn: "Id",
                onDelete: ReferentialAction.Cascade);
        }
    }
}
